@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Document;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One JSON Schema document
 
-our $VERSION = '0.519';
+our $VERSION = '0.520';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -44,9 +44,9 @@ has resource_index => (
   isa => HashRef[my $resource_type = Dict[
       canonical_uri => InstanceOf['Mojo::URL'],
       path => Str,  # always a JSON pointer, relative to the document root
-      specification_version => Str,
+      specification_version => Str, # not an Enum due to module load ordering
       # the vocabularies used when evaluating instance data against schema
-      vocabularies => ArrayRef[ClassName->where(sub { $_->DOES('JSON::Schema::Modern::Vocabulary') })],
+      vocabularies => ArrayRef[my $vocabulary_class_type = ClassName->where(q{$_->DOES('JSON::Schema::Modern::Vocabulary')})],
       slurpy HashRef[Undef],  # no other fields allowed
     ]],
   handles_via => 'Hash',
@@ -101,6 +101,13 @@ has evaluation_configs => (
   default => sub { {} },
 );
 
+# if this document is used as a metaschema, these are the vocabularies used in the metaschema
+has metaschema_vocabulary_classes => (
+  is => 'ro',
+  isa => ArrayRef[$vocabulary_class_type],
+  writer => '_set_metaschema_vocabulary_classes',
+);
+
 around _add_resources => sub {
   my $orig = shift;
   my $self = shift;
@@ -176,6 +183,9 @@ sub BUILD {
 
   # overlay the resulting configs with those that were provided by the caller
   $self->_set_evaluation_configs(+{ %{$state->{configs}}, %{$self->evaluation_configs} });
+
+  $self->_set_metaschema_vocabulary_classes($state->{metaschema_vocabulary_classes})
+    if $state->{metaschema_vocabulary_classes};
 }
 
 1;
@@ -194,7 +204,7 @@ JSON::Schema::Modern::Document - One JSON Schema document
 
 =head1 VERSION
 
-version 0.519
+version 0.520
 
 =head1 SYNOPSIS
 
@@ -259,6 +269,14 @@ This should never need to be set explicitly. This is sometimes populated automat
 creating a document object, depending on the keywords found in the schema, but they will never
 override anything you have already explicitly set.
 
+=head2 metaschema_vocabulary_classes
+
+=for stopwords metaschema
+
+A hashref of classnames that correspond to the vocabularies that should be used when this schema
+document is used as a metaschema for another schema (i.e. in that schema's C<$schema> keyword).
+Derived during construction of the document.
+
 =head1 METHODS
 
 =for Pod::Coverage FOREIGNBUILDARGS BUILDARGS BUILD
