@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Applicator;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Applicator vocabulary
 
-our $VERSION = '0.519';
+our $VERSION = '0.520';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -22,13 +22,12 @@ use namespace::clean;
 with 'JSON::Schema::Modern::Vocabulary';
 
 sub vocabulary {
-  my ($self, $spec_version) = @_;
-  return
-      $spec_version eq 'draft2019-09' ? 'https://json-schema.org/draft/2019-09/vocab/applicator'
-    : $spec_version eq 'draft2020-12' ? 'https://json-schema.org/draft/2020-12/vocab/applicator'
-    : undef;
+  'https://json-schema.org/draft/2019-09/vocab/applicator' => 'draft2019-09',
+  'https://json-schema.org/draft/2020-12/vocab/applicator' => 'draft2020-12';
 }
 
+sub evaluation_order { 1 }
+
 # the keyword order is arbitrary, except:
 # - if must be evaluated before then, else
 # - items must be evaluated before additionalItems
@@ -613,11 +612,11 @@ JSON::Schema::Modern::Vocabulary::Applicator - Implementation of the JSON Schema
 
 =head1 VERSION
 
-version 0.519
+version 0.520
 
 =head1 DESCRIPTION
 
-=for Pod::Coverage vocabulary keywords
+=for Pod::Coverage vocabulary evaluation_order keywords
 
 =for stopwords metaschema
 
